@@ -94,6 +94,18 @@ impl<const W: u16, const H: u16> Qa<W, H> {
         Self { x: X, y: Y }
     }
 
+    /// Return true if self is a corner of the grid.
+    #[inline]
+    pub fn is_corner(&self) -> bool {
+        (self.x == 0 || self.x == W - 1) && (self.y == 0 || self.y == H - 1)
+    }
+
+    /// Return true if self is on the side of the grid.
+    #[inline]
+    pub fn is_side(&self) -> bool {
+        self.x == 0 || self.x == W - 1 || self.y == 0 || self.y == H - 1
+    }
+
     /// Return the corresponding `(u16, u16)` tuple.
     #[inline]
     pub fn tuple(&self) -> (u16, u16) {
@@ -504,8 +516,9 @@ impl fmt::Display for Qr {
 
 /* QrIter: */
 
-/// Iterator for all possible values for the [`Qr`] type used, in
-/// clockwise order.
+/// Iterator for [`Qr`] cardinal and itercardinal directions
+///
+/// Iterate over all possible values of [`Qr`], in clockwise order.
 ///
 /// Example that prints all 4 cardinal directions:
 ///
@@ -566,7 +579,7 @@ impl<const W: u16, const H: u16> ops::Add<Qr> for Qa<W, H> {
 
 /* Grid: a Qa-indexed array *****************************************/
 
-/// A grid is a generic array that can be indexed by a Qa
+/// A grid is a generic array that can be indexed by a [`Qa`]
 ///
 /// We can also interact with specific lines with [`Grid::line`] and
 /// [`Grid::line_mut`], or with the whole underlying array with
@@ -576,6 +589,9 @@ impl<const W: u16, const H: u16> ops::Add<Qr> for Qa<W, H> {
 /// At the moment we have to provide a `SIZE` argument = `WIDTH` *
 /// `HEIGHT`. This value is checked at compile time, but can't be
 /// ellided at the moment, due to rust const generics limitations.
+///
+/// We can use the [`grid_create`] macro to use a [`Qa`] as a source
+/// of these values.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct Grid<T, const WIDTH: u16, const HEIGHT: u16, const SIZE: usize>([T; SIZE]);
 
@@ -828,7 +844,7 @@ impl<T: Copy, const W: u16, const H: u16, const SIZE: usize> iter::FromIterator<
     }
 }
 
-// extend
+// Extend
 
 impl<T, const W: u16, const H: u16, const SIZE: usize> iter::Extend<(Qa<W, H>, T)>
     for Grid<T, W, H, SIZE>
@@ -872,6 +888,62 @@ impl<'a, T: 'a + Copy, const W: u16, const H: u16, const SIZE: usize>
     }
 }
 
+// Display, with helper
+
+fn display_fmt_helper(
+    f: &mut fmt::Formatter<'_>,
+    w: u16,
+    h: u16,
+    mut it: impl Iterator<Item = String>,
+) -> fmt::Result {
+    // Max digits for column numbers:
+    let ndigits_x = format!("{}", w).len();
+    // Max digits for line numbers:
+    let ndigits_y = format!("{}", h).len();
+    // Column labels as a vec of vec of chars, which we will
+    // output vertically:
+    let str_x = (0..w)
+        .map(|i| {
+            format!("{:width$}", i, width = ndigits_x)
+                .chars()
+                .collect::<Vec<_>>()
+        })
+        .collect::<Vec<_>>();
+    // Print the column labels; we do this both on top and on the
+    // bottom of the grid:
+    let headerfooter = |f: &mut fmt::Formatter<'_>| {
+        for digit in 0..ndigits_x {
+            f.write_fmt(format_args!("{:width$} ", "", width = ndigits_y))?;
+            for cell in str_x.iter() {
+                f.pad(&cell[digit].to_string())?;
+            }
+            f.write_str("\n")?;
+        }
+        Ok(())
+    };
+    headerfooter(f)?;
+    // Print the cells of the grid, qa by qa, controlling for new lines:
+    let mut last_y = h;
+    for y in 0..h {
+        for _x in 0..w {
+            if y != last_y {
+                // We are printing a new line:
+                if y > 0 {
+                    // End the current line before first:
+                    f.write_str("\n")?;
+                }
+                // Print the line number as a label:
+                f.write_fmt(format_args!("{:width$} ", y, width = ndigits_y))?;
+                last_y = y;
+            }
+            let s = it.next().unwrap();
+            f.pad(&s)?;
+        }
+    }
+    f.write_str("\n")?;
+    headerfooter(f)
+}
+
 /// Pretty-printer [`Grid`] display implementation
 ///
 /// The [`Display`](std::fmt::Display) implementation of grid was made
@@ -882,50 +954,235 @@ impl<T: fmt::Display, const W: u16, const H: u16, const SIZE: usize> fmt::Displa
     for Grid<T, W, H, SIZE>
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        // Max digits for column numbers:
-        let ndigits_x = format!("{}", W).len();
-        // Max digits for line numbers:
-        let ndigits_y = format!("{}", H).len();
-        // Column labels as a vec of vec of chars, which we will
-        // output vertically:
-        let str_x = (0..W)
-            .map(|i| {
-                format!("{:width$}", i, width = ndigits_x)
-                    .chars()
-                    .collect::<Vec<_>>()
-            })
-            .collect::<Vec<_>>();
-        // Print the column labels; we do this both on top and on the
-        // bottom of the grid:
-        let headerfooter = |f: &mut fmt::Formatter<'_>| {
-            for digit in 0..ndigits_x {
-                f.write_fmt(format_args!("{:width$} ", "", width = ndigits_y))?;
-                for cell in str_x.iter() {
-                    f.pad(&cell[digit].to_string())?;
-                }
-                f.write_str("\n")?;
-            }
-            Ok(())
+        display_fmt_helper(f, W, H, self.iter().map(|v| format!("{}", v)))
+    }
+}
+
+/* Gridbool: a grid of booleans optimized for space *****************/
+
+/// Space-optimized grid of booleans using bitmaps
+///
+/// `Gridbool` uses an array of u32 to implement a [`Qa`]-indexable
+/// grid of booleans, balancing space and performance.
+///
+/// At the moment we need to provide the number of u32 WORDS to
+/// use due to rust generic const limitations. We are able to check
+/// that the value is appropriate, though.
+///
+/// We can use the [`gridbool_create`] macro to use a [`Qa`] as a
+/// source of these values.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct Gridbool<const WIDTH: u16, const HEIGHT: u16, const WORDS: usize>([u32; WORDS]);
+
+/// Helper macro for Gridbool type creation.
+///
+/// More often than not we want to create a grid form an associated
+/// [`Qa`] type. This macros makes the process easier.
+///
+/// Example usage:
+/// ```
+/// type Qa = sqrid::Qa<3, 3>;
+/// type Gridbool = sqrid::gridbool_create!(Qa);
+/// ```
+#[macro_export]
+macro_rules! gridbool_create {
+    ($qa: ty) => {
+        sqrid::Gridbool<{ <$qa>::WIDTH }, { <$qa>::HEIGHT },
+                        { (((<$qa>::WIDTH - 1) * (<$qa>::HEIGHT - 1)) / 32 + 1) as usize }>
+    };
+}
+
+impl<const W: u16, const H: u16, const WORDS: usize> Gridbool<W, H, WORDS> {
+    // Create the _ASSERTS constant to check W * H == SIZE
+    // We have to instantiate it in all low-level constructors to
+    // actually perform the check.
+    impl_assert!(_ASSERTS;
+                 // WORDS is big enough to hold all bits:
+                 W as usize * H as usize <= WORDS * 32 &&
+                 // WORDS is not bigger than necessary:
+                 W as usize * H as usize > WORDS * 32 - 32);
+    // Used in creation:
+    const WORD_FALSE: u32 = 0;
+    const WORD_TRUE: u32 = 0xFFFFFFFF;
+    // These are used to iterate over references:
+    const TRUE: bool = true;
+    const FALSE: bool = false;
+
+    /// Const Gridbool filled with `true` values.
+    pub const ALL_TRUE: Self = Self::repeat(true);
+    /// Const Gridbool filled with `false` values.
+    pub const ALL_FALSE: Self = Self::repeat(false);
+
+    /// Create a Gridbool filled with the provided `value`.
+    #[inline]
+    pub const fn repeat(value: bool) -> Self {
+        let _ = Self::_ASSERTS;
+        let v = if value {
+            Self::WORD_TRUE
+        } else {
+            Self::WORD_FALSE
         };
-        headerfooter(f)?;
-        // Print the cells of the grid, qa by qa, controlling for new lines:
-        let mut last_y = H;
-        for qa in Qa::iter() {
-            if qa.y != last_y {
-                // We are printing a new line:
-                if qa.y > 0 {
-                    // End the current line before first:
-                    f.write_str("\n")?;
-                }
-                // Print the line number as a label:
-                f.write_fmt(format_args!("{:width$} ", qa.y, width = ndigits_y))?;
-                last_y = qa.y;
-            }
-            let s = format!("{}", self[qa]);
-            f.pad(&s)?;
+        Gridbool([v; WORDS])
+    }
+
+    #[inline]
+    fn byte_bit(i0: impl Into<usize>) -> (usize, u32) {
+        let i = i0.into();
+        let byte = i / 32;
+        let bit = 0x80000000 >> (i % 32);
+        (byte, bit)
+    }
+
+    /// Set the provided [`Qa`] position to `true`.
+    #[inline]
+    pub fn set_t(&mut self, qaref: impl Borrow<Qa<W, H>>) {
+        let (byte, bit) = Self::byte_bit(qaref.borrow());
+        self.0[byte] |= bit;
+    }
+
+    /// Set the provided [`Qa`] position to `false`.
+    #[inline]
+    pub fn set_f(&mut self, qaref: impl Borrow<Qa<W, H>>) {
+        let (byte, bit) = Self::byte_bit(qaref.borrow());
+        self.0[byte] &= !bit;
+    }
+
+    /// Set the provided [`Qa`] position to `value`.
+    #[inline]
+    pub fn set(&mut self, qaref: impl Borrow<Qa<W, H>>, value: bool) {
+        if value {
+            self.set_t(qaref)
+        } else {
+            self.set_f(qaref)
         }
-        f.write_str("\n")?;
-        headerfooter(f)
+    }
+
+    /// Return the value at position [`Qa`].
+    #[inline]
+    pub fn get(&self, qaref: impl Borrow<Qa<W, H>>) -> bool {
+        let (byte, bit) = Self::byte_bit(qaref.borrow());
+        self.0[byte] & bit != 0
+    }
+
+    /// Consume self and returns the inner bitmap.
+    #[inline]
+    pub fn into_inner(self) -> [u32; WORDS] {
+        self.0
+    }
+
+    /// Return a reference to the inner bitmap; useful for testing.
+    #[inline]
+    pub fn as_inner(&self) -> &[u32; WORDS] {
+        &self.0
+    }
+
+    /// Return a mut reference to the inner bitmap; useful for testing.
+    #[inline]
+    pub fn as_inner_mut(&mut self) -> &mut [u32; WORDS] {
+        &mut self.0
+    }
+
+    /// Iterate over all `true`/`false` values in the `Gridbool`.
+    #[inline]
+    pub fn iter(&self) -> impl iter::Iterator<Item = bool> + '_ {
+        (0..(W * H)).map(move |i| {
+            let (byte, bit) = Self::byte_bit(i);
+            self.0[byte] & bit != 0
+        })
+    }
+
+    /// Iterate over all coordinates and corresponding `true`/`false` values.
+    #[inline]
+    pub fn iter_qa(&self) -> impl iter::Iterator<Item = (Qa<W, H>, bool)> + '_ {
+        Qa::<W, H>::iter().map(move |qa| (qa, self[qa]))
+    }
+
+    /// Iterate over all `true` coordinates the `Gridbool`.
+    #[inline]
+    pub fn iter_t(&self) -> impl Iterator<Item = Qa<W, H>> + '_ {
+        Qa::<W, H>::iter().filter(move |qa| self[qa])
+    }
+
+    /// Iterate over all `false` coordinates the `Gridbool`.
+    #[inline]
+    pub fn iter_f(&self) -> impl Iterator<Item = Qa<W, H>> + '_ {
+        Qa::<W, H>::iter().filter(move |qa| !self[qa])
+    }
+
+    /// Take a [`Qa`] iterator and set all corresponding values to `true`.
+    #[inline]
+    pub fn set_iter_t<AQA>(&mut self, qaiter: impl Iterator<Item = AQA>)
+    where
+        AQA: Borrow<Qa<W, H>>,
+    {
+        for qa in qaiter {
+            self.set_t(qa);
+        }
+    }
+
+    /// Take a [`Qa`] iterator and set all corresponding values to `false`.
+    #[inline]
+    pub fn set_iter_f<AQA>(&mut self, qaiter: impl Iterator<Item = AQA>)
+    where
+        AQA: Borrow<Qa<W, H>>,
+    {
+        for qa in qaiter {
+            self.set_f(qa);
+        }
+    }
+}
+
+impl<const W: u16, const H: u16, const WORDS: usize> Default for Gridbool<W, H, WORDS> {
+    fn default() -> Self {
+        Self::ALL_FALSE
+    }
+}
+
+// Indexing
+
+impl<AQA, const W: u16, const H: u16, const WORDS: usize> ops::Index<AQA> for Gridbool<W, H, WORDS>
+where
+    AQA: Borrow<Qa<W, H>>,
+{
+    type Output = bool;
+    #[inline]
+    fn index(&self, aqa: AQA) -> &Self::Output {
+        // Trick to be able to return reference to boolean as required
+        // by trait:
+        if self.get(aqa) {
+            &Self::TRUE
+        } else {
+            &Self::FALSE
+        }
+    }
+}
+
+// from_iter
+
+impl<AQA, const W: u16, const H: u16, const WORDS: usize> iter::FromIterator<AQA>
+    for Gridbool<W, H, WORDS>
+where
+    AQA: Borrow<Qa<W, H>>,
+{
+    #[inline]
+    fn from_iter<I>(iter: I) -> Self
+    where
+        I: iter::IntoIterator<Item = AQA>,
+    {
+        let mut gb = Gridbool::<W, H, WORDS>::ALL_FALSE;
+        gb.set_iter_t(iter.into_iter());
+        gb
+    }
+}
+
+impl<const W: u16, const H: u16, const WORDS: usize> fmt::Display for Gridbool<W, H, WORDS> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        display_fmt_helper(
+            f,
+            W,
+            H,
+            self.iter().map(|b| (if b { "#" } else { "." }).to_string()),
+        )
     }
 }
 
